<?php
/*
* Plugin Name: Go2wpForo
* Plugin URI: http://wpforo.com
* Description: WordPress forum plugins to wpForo migration tool.
* Author: gVectors Team (R. Hovhannisyan)
* Author URI: http://gvectors.com/
* Version: 1.0.0
*/

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

define('MG2WPFORO_DIR', rtrim( str_replace( '//', '/', dirname(__FILE__)), '/'));
define('MG2WPFORO_URL', rtrim( WP_PLUGIN_URL . '/' . plugin_basename(dirname(__FILE__)), '/'));
define('MG2WPFORO_FOLDER', rtrim( plugin_basename(dirname(__FILE__)), '/'));
define('MG2WPFORO_BASENAME', plugin_basename( __FILE__ ));

class Migrate2wpForo{
	public $wpforo;
	private $db;
	private static $cache = array();
	public $itempercycle = 0;
	
	public function __construct($wpforo){
		$this->wpforo = $wpforo;
		$this->db = $wpforo->db;
		
		$this->setup();
	}
	
	private function setup(){
		set_time_limit(0);
		ini_set('max_execution_time', 0);
		ini_set('memory_limit', '-1');
		$this->db->hide_errors();
		register_deactivation_hook(__FILE__, array(&$this, 'mg2wpforo_deactivation'));
		add_action( 'admin_menu', array(&$this, 'mg2wpforo_add_menu'), 40 );
	}
	
	public function mg2wpforo_add_menu(){
		add_menu_page( 'Go2wpForo', 'Go2wpForo', 'read', 'Go2wpForo', array(&$this, 'mg2wpforo_main_page'), 'dashicons-migrate' );
		
		$boards = glob( MG2WPFORO_DIR . '/boards/*.php' );
		if( !empty($boards) ){
			foreach( $boards as $board ){
				$board = basename($board, '.php');
				add_submenu_page( 'Go2wpForo', $board, '&nbsp;'.strtoupper($board).'&nbsp;&raquo;', 'read',   $board.'-mg2wpforo', array(&$this, 'mg2wpforo_board_page') );	
			}
		}
	}
	
	public function mg2wpforo_main_page(){ ?>
		<h1 style="text-align:center; font-weight:500; font-size:26px; padding:20px;">Welcome to Go2wpForo 1.0.0 Tool</h1>
		<center><img src="<?php echo MG2WPFORO_URL ?>/mg2wpforo.png" style="border:1px solid #ccc; height:110px;"/></center>
        <br/>
        <div class="wrap" style="text-align:center;">
        <table width="100%" border="0" cellspacing="0" cellpadding="0" style="margin:10px auto;">
          <tbody>
          	<tr>
              <td style="text-align:left; background:#fff; padding:20px; font-size:14px; line-height:20px;" colspan="3">
              	We do our best to improve this free migration tool as much as possible, however that's very hard to make migration tool for all kind and size of forums. 
                This tool is designed for default forum components like &laquo;Forums&raquo;, &laquo;Topics&raquo;, &laquo;Posts&raquo;, &laquo;Users&raquo;. This will not migrate specific data like &laquo;PMs&raquo;, &laquo;Likes&raquo;, &laquo;Polls&raquo;, &laquo;User Reputation&raquo;, etc... 
                So if you found some issue with default components migration, please open a support topic in our 
                <a href="http://wpforo.com/community/migrate-to-wpforo-from-other-forum-plugins/" target="_blank">support forum</a>.  
                We'll try to help you. <br/>
                For deeper support and for extra data(component) migration you should contact to gVectors Team specialists at <a href="http://gvectors.com/contact-us/" target="_blank">http://gvectors.com/contact-us/</a>. 
                They provide with paid migration services from all WordPress forum plugins to wpForo. <br/>
                If you need stand-alone forum to wpForo migration, e.g.: 
                <a href="http://profprojects.com/migrate-vbulletin-forum-to-wpforo/" target="_blank">vBulletin</a>, 
                <a href="http://profprojects.com/migrate-xenforo-to-wpforo/" target="_blank">XenForo</a>, 
                <a href="http://profprojects.com/migrate-phpbb-to-wpforo/" target="_blank">phpBB</a>, etc... we recommend 
                <a href="http://profprojects.com/" target="_blank">ProfProjects Migration Services</a>. 
                We've already instructed all specialists of this company for wpForo migration projects, they are our trusted partners.
              </td>
            </tr>
            </tbody>
           </table>
        </div>
        <br>
        <div class="wrap" style="text-align:center;">
        <table width="70%" border="0" cellspacing="0" cellpadding="0" style="margin:10px auto;">
          <tbody>
          	<tr>
              <td style="text-align:center; background:#fff; padding:10px;" colspan="3"><h3>Available Migrations</h3></td>
            </tr>
            <tr>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/bbpress.png" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;">bbPress to wpForo</a></p>
              </td>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/simplepress.png" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=simplepress-mg2wpforo" style="text-decoration:none;">Simple:Press to wpForo</a></p>
              </td>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/asgaros.png" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=anspress-mg2wpforo" style="text-decoration:none;">Asgaros to wpForo</a></p>
              </td>
            </tr>
            <tr>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/anspress.png" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;">AnsPress to wpForo</a></p>
              </td>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/dwqa.jpg" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=simplepress-mg2wpforo" style="text-decoration:none;">DW Q&amp;A to wpForo</a></p>
              </td>
              <td style="text-align:center; background:#fff; padding:30px 10px;">
              <a href="admin.php?page=bbpress-mg2wpforo" style="text-decoration:none;"><img src="<?php echo MG2WPFORO_URL ?>/images/wpqa.png" style="border:1px solid #ccc; height:70px;"/></a>
              <p style="padding:0; margin:0; font-size:16px;"><a href="admin.php?page=anspress-mg2wpforo" style="text-decoration:none;">WP Q&amp;A to wpForo</a></p>
              </td>
            </tr>
          </tbody>
        </table>
		</div>
		<?php
	}
	
	public function mg2wpforo_board_page(){
		if( isset($_GET['page']) && $board = trim(basename($_GET['page'], '-mg2wpforo')) ) mg2wpforo_init_action($board);
	}
	
	public function mg2wpforo_deactivation(){
		if( version_compare($this->db->db_version(), '5.6.4', '>=') ){
			$sql = "ALTER TABLE " . $this->db->prefix . "wpforo_topics ENGINE = InnoDB";
			$this->db->query($sql);
			$sql = "ALTER TABLE " . $this->db->prefix . "wpforo_posts ENGINE = InnoDB";
			$this->db->query($sql);
		}
	}

	public function show_statistics_table($board){
		$statistic = $this->wpforo->get_statistic();
		?>
		<table align="center" cellpadding="0" cellspacing="0" width="100%" height="200">
			<tr align="center"><th>Objects</th><th><?php echo strtoupper($board) ?></th><th>wpForo</th></tr>
			<tr align="center">
				<td>Users:</td>
				<td><b><?php echo $this->$board->get_users_count() ?></b></td>
				<td><b><?php echo $statistic['members'] ?></b></td>
			</tr>
			<tr align="center">
				<td>Forums:</td>
				<td><b><?php echo $this->$board->get_forums_count() ?></b></td>
				<td><b><?php echo $statistic['forums'] ?></b></td>
			</tr>
			<tr align="center">
				<td>Topics:</td>
				<td><b><?php echo $this->$board->get_topics_count() ?></b></td>
				<td><b><?php echo $statistic['topics'] ?></b></td>
			</tr>
			<tr align="center">
				<td>Posts:</td>
				<td><b><?php echo $this->$board->get_posts_count() ?></b></td>
				<td><b><?php echo $statistic['posts'] ?></b></td>
			</tr>
		</table>
		<?php
	}
	
	public function get_progress_percent($board){
		$items_count = $this->$board->get_forums_count() + $this->$board->get_posts_count();
		if(!$items_count) return 0;
		$percent = ceil( $this->get_log_count() * 100 / $items_count ); 
		if(!$percent || $percent < 1) $percent = 1;
		if($percent > 100) $percent = 100;
		return $percent;
	}
	
	public function show_progress_bar($board){ 
		$percent = $this->get_progress_percent($board); ?>
		<div style="position: relative; background-color: white; width: 650px; height: 50px; border: 1px solid grey; padding: 2px; text-align: left;">
			<span style="color: rgb(13, 110, 62); font-size: 2.5em; position: absolute; left: 47%; top: 32%;"><?php echo $percent ?>%</span>
			<div style="background: rgba(79,240,119,1);
				background: -moz-linear-gradient(top, rgba(79,240,119,1) 0%, rgba(87,233,100,1) 100%);
				background: -webkit-gradient(left top, left bottom, color-stop(0%, rgba(79,240,119,1)), color-stop(100%, rgba(87,233,100,1)));
				background: -webkit-linear-gradient(top, rgba(79,240,119,1) 0%, rgba(87,233,100,1) 100%);
				background: -o-linear-gradient(top, rgba(79,240,119,1) 0%, rgba(87,233,100,1) 100%);
				background: -ms-linear-gradient(top, rgba(79,240,119,1) 0%, rgba(87,233,100,1) 100%);
				background: linear-gradient(to bottom, rgba(79,240,119,1) 0%, rgba(87,233,100,1) 100%);
				filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#4ff077', endColorstr='#57e964', GradientType=0 ); 
				height: 100%; width: <?php echo $percent ?>%"></div>
		</div>
		<?php
	}

	public function add_log($obj, $old_id, $new_id){
		return $this->db->query("INSERT INTO ".$this->db->prefix."mg2wpforo SET obj = '$obj', old_id = $old_id, new_id = $new_id");
	}

	public function clear_log(){
		return $this->db->query("TRUNCATE ".$this->db->prefix."mg2wpforo");
	}
	
	public function get_log_count(){
		if( $count = $this->db->get_var("SELECT COUNT(*) FROM ".$this->db->prefix."mg2wpforo") ) return $count;
		return 0;
	}
	
	public function get_last_old_id($obj){
		return $this->db->get_var("SELECT old_id FROM ".$this->db->prefix."mg2wpforo WHERE obj = '$obj' ORDER BY 1 DESC LIMIT 1");
	}
	
	public function get_old_id($obj, $new_id){
		return $this->db->get_var("SELECT old_id FROM ".$this->db->prefix."mg2wpforo WHERE obj = '$obj' AND new_id = $new_id");
	}
	
	public function get_new_id($obj, $old_id){
		if( isset(self::$cache[$obj][$old_id]) ) return self::$cache[$obj][$old_id];
		$sql = "SELECT new_id FROM ".$this->db->prefix."mg2wpforo WHERE obj = '$obj' AND old_id = $old_id";
		return self::$cache[$obj][$old_id] = $this->db->get_var($sql);
	}
	
	public function do_forums($board){
		if( !isset($_GET['layout']) || !($cat_layout = intval($_GET['layout'])) ) $cat_layout = 1;
		if( method_exists($this->$board, 'get_extracats') ){
			if( $extracats = $this->$board->get_extracats() ){
				foreach( $extracats as $extracat ){
					$new_id = 0;
					if( !$this->get_new_id('extracat', $extracat['old_id']) ){
						$extracat['cat_layout'] = $cat_layout;
						if( $f_id = $this->wpforo->forum->add($extracat) ) $new_id = $f_id;
					}
					$this->add_log('extracat', $extracat['old_id'], $new_id);
				}
			}
		}else{
			$fid = $this->wpforo->forum->add( array('title' => 'Migrated Forums', 'cat_layout' => $cat_layout) );
		}
		
		if( $forums = $this->$board->get_forums() ){
			foreach( $forums as $forum ){
				if( method_exists($this->$board, 'get_extracats') ) $fid = 0;
				if( !$fid && isset($forum['extracat']) && ($forum['extracat'] = intval($forum['extracat'])) ) $fid = $this->get_new_id('extracat', $forum['extracat'] );
				$new_id = 0;
				if( !$this->get_new_id('forum', $forum['old_id']) ){
					$forum['parentid'] = ( isset($forum['parentid']) && $forum['parentid'] ? $this->get_new_id('forum', $forum['parentid']) : $fid );
					$forum['cat_layout'] = $cat_layout;
					if( $f_id = $this->wpforo->forum->add($forum) ) $new_id = $f_id;
				}
				$this->add_log('forum', $forum['old_id'], $new_id);
			}
		}
		$this->wpforo->notice->clear();
	}
	
	public function do_topics($board){
		if( !$lastid = $this->get_last_old_id('topic') ) $lastid = 0;
		if( $topics = $this->$board->get_topics($lastid) ){
			foreach( $topics as $topic ){
				$new_id = 0;
				if( !$this->get_new_id('topic', $topic['old_id']) ){
					$topic['forumid'] = $this->get_new_id('forum', $topic['forumid']);
					if( $t_id = $this->wpforo->topic->add($topic) ) $new_id = $t_id;
				}
				$this->add_log('topic', $topic['old_id'], $new_id);
			}
			$this->wpforo->notice->clear();
			return FALSE;
		}
		$this->wpforo->notice->clear();
		return TRUE;
	}
	
	public function do_replies($board){
		if( !$lastid = $this->get_last_old_id('reply') ) $lastid = 0;
		if( $replies = $this->$board->get_replies($lastid) ){
			foreach( $replies as $reply ){
				$new_id = 0;
				if( !$this->get_new_id('reply', $reply['old_id']) ){
					$reply['topicid'] = $this->get_new_id('topic', $reply['topicid']);
					if( $r_id = $this->wpforo->post->add($reply) ) $new_id = $r_id;
				}
				$this->add_log('reply', $reply['old_id'], $new_id);
			}
			$this->wpforo->notice->clear();
			return FALSE;
		}
		if( method_exists($this->$board, 'get_comments') ) return $this->do_comments($board);
		$this->wpforo->notice->clear();
		return TRUE;
	}
	
	private function do_comments($board){
		if( !$lastid = $this->get_last_old_id('comment') ) $lastid = 0;
		if( $comments = $this->$board->get_comments($lastid) ){
			foreach( $comments as $comment ){
				$new_id = 0;
				if( !$this->get_new_id('comment', $comment['old_id']) ){
					$comment['topicid'] = $this->get_new_id('topic', $comment['topicid']);
					$parentid = $this->get_new_id('reply', $comment['parentid']);
					if(!$parentid){
						if( $parentid = $this->get_new_id('topic', $comment['parentid']) ){
							if( $topic = $this->wpforo->topic->get_topic($parentid) ) $parentid = $topic['first_postid'];
						}
					}
					$comment['parentid'] = $parentid;
					if( $c_id = $this->wpforo->post->add($comment) ) $new_id = $c_id;
				}
				$this->add_log('comment', $comment['old_id'], $new_id);
			}
			$this->wpforo->notice->clear();
			return FALSE;
		}
		$this->wpforo->notice->clear();
		return TRUE;
	}
}
	
require_once("includes/functions.php");
	
?>